// calc cross section
// and 
// convert LAB to CM 
// ver.1.0.  2007/12/12 
// ver.2.0.  2007/12/13
// ver.2.1.  2007/12/14
// ver.3.0.  2007/12/20
// ver.3.5.  2007/12/20
// ver.5.0.  2008/01/06
// ver.5.5.  2008/01/15
// 			*texfile out
// ver.6.0.
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "defines.h"
#include "commonfunction.h"

void do_calc(char *buf);
void make_pltfile(char *buf);
void do_draw(char *buf);
int check_graph(char *buf);
void draw_scale(FILE *datafile, int ymin, int unitlength, int digit);
void draw_axis(FILE *datafile, int deg, int unitlength, int digit);
double get_target_number(int target, int nuclei, int position);
void make_tex_file(char *title);

FILE *texout; // tex output
int check_first = 0;

int main(int argc, char *argv[])
{
	FILE *fp;
	char buf[BUFSIZE], texfiletitle[BUFSIZE];

	fp = fopen(argv[1], "r");
	if (fp == NULL) {
		printf("cannot open file(1)\n");
		return;
	}
	// tex template
	make_tex_file(argv[1]);
	
    // calculation 
	while (1) {
		if (fgets(buf,sizeof(buf),fp) == NULL)
			break;

		do_calc(buf);
	}
	fclose(fp);
	
	// draw graph 
	fp = fopen(argv[1], "r");
	if (fp == NULL) {
		printf("cannot open file(2)\n");
	}

	while (1) {
		if (fgets(buf,sizeof(buf),fp) == NULL)
			break;
		
		do_draw(buf);
	}
	fclose(fp);

 	fprintf(texout, "\\bottomrule\n\\end{tabular}\n\\end{center}\n\\end{table}\n\n");
	fprintf(texout, "\\end{document}\n");
	fclose(texout);
}

void do_calc(char *buf)
{
	char runname[BUFSIZE], title[BUFSIZE], graphtitlecm[BUFSIZE],graphtitlelab[BUFSIZE];
	int readitem, i, a[4], z[4], sw, target, nuclei, position;
	double mass[4], massex[4], Q, A, B, gammafactor, ECM, Ex,E;
	double channel, channelerr; // not use this program
	double deg, degCM, gross, grosserr, scalerCount, incidentcount;
	double crosssec, dcrosssec, crosssecCM, dcrosssecCM, targetNumber;
	FILE *fileout;

	// read parameters 
	readitem = sscanf(buf,"%s %s %lf %lf %lf %lf %lf %lf",
			runname, title, &deg, &channel, &channelerr,
			&gross, &grosserr, &scalerCount);

	if (readitem != 8)
		return;

	// get atomic number
	if (!get_atomic_number(title, a,z)) {
		return;
	}
	// get parameter
	if (!get_parameter(runname,&E,&position,&target,&scalerCount)) {
		return;
	}
	// last Ex, nuclei
	Ex = get_ex_toi(title);
	nuclei = get_nuclei_number(a[1],target);

	// calc cross section 
	targetNumber = get_target_number(target, nuclei, position);
	incidentcount = scalerCount*COULPRPLSE/ELECHARGE;
	crosssec = gross/(incidentcount*targetNumber*dOMEGA)*1e24;
	dcrosssec = sqrt(gross)/(incidentcount*targetNumber*dOMEGA)*1e24;

	// convert lab to cm frame 
	for (i = 0; i < 4; i++) {
		massex[i] = calc_massex(a[i], z[i]);
		mass[i] = calc_mass(a[i],z[i],massex[i]);
	}
	Q = massex[0]+massex[1]-massex[2]-massex[3];
	ECM = (mass[1]/(mass[0]+mass[1]))*E;
	gammafactor = sqrt(mass[0]*mass[2]/(mass[1]*mass[3])*ECM/(ECM+Q-Ex));

	degCM = deg + (180/PI)*asin(gammafactor*sin(deg*PI/180));

	A = 1 + gammafactor*cos(degCM*PI/180);
	B = 1 + 2*gammafactor*cos(degCM*PI/180)+gammafactor*gammafactor;
	crosssecCM = A/(exp((3.0e0/2.0e0)*log(B)))*crosssec;
	dcrosssecCM = A/(exp((3.0e0/2.0e0)*log(B)))*dcrosssec;

	// check output file already exits ? 
	sw = check_graph(title);

	// sw == 1 means need new file 
	if (sw)
		fileout = fopen(title, "w");
	else
		fileout = fopen(title, "a");

	if (fileout == NULL)
		 return;

	if (sw == 1) { // make new output file 
	 	 // template
		 fprintf(fileout, "#%s cross section [barn]\n",title);
		 fprintf(fileout, "#E(LAB) = %3.2f, E(CM) = %3.2f, Ex = %3.2f, Q-value = %3.2f\n", E, ECM, Ex, Q);
		 fprintf(fileout, "#run, gross, FC, deg_LAB, crssc_LAB, error, deg_CM, crssc_CM, error\n");
		 // gnuplot plt file 
		 make_pltfile(title);
		 // tex file
		 if (check_first == 1) {
		 	fprintf(texout, "\\bottomrule\n\\end{tabular}\n\\end{center}\n\\end{table}\n\n");
		}
		check_first = 1;
		 strcpy(graphtitlelab, title);
		 strcpy(graphtitlecm, title);
		 fprintf(texout, "\n\\newpage\n\n\\begin{figure}[H]\n\\centering\n");
		 fprintf(texout, "\\includegraphics[width=14cm,clip]{%s}\n\\end{figure}\n", 
				 strcat(graphtitlecm, "CM.eps"));
		 fprintf(texout, "\\begin{figure}[H]\n\\centering\n");
		 fprintf(texout, "\\includegraphics[width=14cm,clip]{%s}\\end{figure}\n\n", 
				 strcat(graphtitlelab, "LAB.eps"));
	 	 fprintf(texout, "\\newpage\\begin{table}[H]\n\\begin{center}\n\\begin{tabular}{l|ll|lll|lll}\n");
		 fprintf(texout, "\\multicolumn{9}{c}{%s cross section [barn]}\\\\\n", title);
		 fprintf(texout, "\\toprule\n");
		 fprintf(texout, "\\multicolumn{9}{c}{E(LAB) = %3.2f, E(CM) = %3.2f, Ex = %3.2f, Q-value = %3.2f}\\\\\n", E, ECM, Ex, Q);
		 fprintf(texout, "\\midrule\n");
		 fprintf(texout, "run & gross & FC & deg(LAB) & crssc(LAB) & error & deg(CM) & crssc(CM) & error\\\\\n");
		 fprintf(texout, "\\midrule\n");
	}
	fprintf(fileout, "%s %4.3le %4.3le %3.1f %4.3le %4.3le %3.1f %4.3le %4.3le\n", 
			 runname, gross, scalerCount,deg, crosssec, dcrosssec, degCM, crosssecCM, dcrosssecCM);

	fprintf(texout, "%s & %4.3le & %4.3le & %3.1f & %4.3le & %4.3le & %3.1f & %4.3le & %4.3le\\\\\n", 
			 runname, gross, scalerCount,deg, crosssec, dcrosssec, degCM, crosssecCM, dcrosssecCM);

	fclose(fileout);
}

void make_pltfile(char *buf)
{
	FILE *fileout;
	char datatitle[BUFSIZE], outputfile[BUFSIZE], datatitlecm[BUFSIZE],datatitlelab[BUFSIZE];
	strcpy(datatitle, buf);
	strcpy(datatitlecm, buf);
	strcpy(datatitlelab, buf);
	strcpy(outputfile,buf);
	//fileout = fopen(strcat(outputfile,".plt"),"w");
	fileout = fopen("makegraph.plt","a");
	if (fileout == NULL) {
		return;
	}
    // template 
	// cm frame
	fprintf(fileout, "\n#%s CM\n", datatitle);
	fprintf(fileout, "set logscale y\nset style line 1 pointtype 3 pointsize 2\n");
	fprintf(fileout, "set ylabel \"d{/Symbol s}/d{/Symbol W}[barn/sr]\"\n");
	fprintf(fileout, "set xlabel \"{/Symbol Q}(CM)\"\n");
	fprintf(fileout, "plot \"%s\" u 7:8:9 w yerror ls 1 title \"%s\"\n", datatitle, 
			strcat(datatitlecm,"CM"));
	fprintf(fileout, "set terminal postscript enhanced color eps\n");
	fprintf(fileout, "set output \"%s\"\n", strcat(datatitlecm,".eps"));
	fprintf(fileout, "replot\n");
	// lab frame
	fprintf(fileout, "\n#%s LAB\n", datatitle);
	fprintf(fileout, "set logscale y\nset style line 1 pointtype 3 pointsize 2\n");
	fprintf(fileout, "set ylabel \"d{/Symbol s}/d{/Symbol W}[barn/sr]\"\n");
	fprintf(fileout, "set xlabel \"{/Symbol q}(LAB)\"\n");
	fprintf(fileout, "plot \"%s\" u 4:5:6 w yerror ls 1 title \"%s\"\n", datatitle, 
			strcat(datatitlelab,"LAB"));
	fprintf(fileout, "set terminal postscript enhanced color eps\n");
	fprintf(fileout, "set output \"%s\"\n", strcat(datatitlelab,".eps"));
	fprintf(fileout, "replot\n");

	fclose(fileout);
}

void do_draw(char *buf)
{
	int readitem, sw = 0,i,j, degmin=DEGMAX,degmax=0, ymax, ymin, unitlength, digit, count = 0;
	char runname[BUFSIZE], outputfile[BUFSIZE], buffer[BUFSIZE], tmp[BUFSIZE];
	char graphtitle[BUFSIZE];
	FILE *datafile;
	double degLAB, degCM, crsscLAB, errLAB, crsscCM, errCM, crsscMAX = 0, crsscMIN = 1e10, points[DEGMAX];

	readitem = sscanf(buf, "%s %s", runname, outputfile);

	if (readitem != 2) return;
	// check outputfile's graph already drawn ? 
	if (check_graph(outputfile)) {
		return;
	}

	// read data 
	datafile = fopen(outputfile,"r");
	if (datafile == NULL) {
		return;
	}

	// initialize points file 
	for (i=0;i<DEGMAX;i++) { points[i] = 0; }

	i = 0;
	while (1) {
		if (fgets(buffer,sizeof(buffer),datafile) == NULL)
			break;
		// read parameter
		// run-name, deg_LAB, crssc_LAB, error, deg_CM, crssc_CM, error 
		readitem = sscanf(buffer, "%s %lf %le %le %lf %le %le", 
			 runname, &degLAB, &crsscLAB, &errLAB, &degCM, &crsscCM, &errCM);

		if (readitem != 7) {
			continue;
		}

		// get deg max and min 
		if ((int)(degCM+0.5) < degmin) degmin = (int)(degCM+0.5);
		if ((int)(degCM+0.5) > degmax) degmax = (int)(degCM+0.5);

		points[(int)(degCM+0.5)] = crsscCM;
		// get max and min 
		if (crsscCM > crsscMAX) crsscMAX = crsscCM;
		if (crsscCM < crsscMIN) crsscMIN = crsscCM;
	}
	fclose(datafile);

	// draw graph 
	datafile = fopen(outputfile, "a");
	if (datafile == NULL) {
		printf("cannot open file(4)\n");
		return;
	}
	// get draw range(deg) min and max 
	if (degmin - 5 < 0) { degmin = 0; }
	else { degmin = degmin - 5; }
	if (degmax + 5 > DEGMAX) { degmax = DEGMAX; }
	else { degmax = degmax + 5; }
	// get draw range(cross section) min and max 
	ymax = ceil(log10(crsscMAX));
	ymin = floor(log10(crsscMIN));

	digit = abs(ymax - ymin);
	unitlength = (int)(LINEMAX / digit);
	// draw graph 
	for(i = degmin; i < degmax; i++) {
		// draw template 
		if (i == degmin) {
			fprintf(datafile, "#graph\n");
			draw_scale(datafile,ymin,unitlength, digit);
			draw_axis(datafile,i,unitlength,digit);
			continue;
		}

		// end template 
		if (i == degmax -1) {
			draw_axis(datafile,i,unitlength,digit);
			draw_scale(datafile,ymin,unitlength,digit);
			continue;
		}

		fprintf(datafile, "#      %3d+", i);
		for (j = 0; j < unitlength*digit - 1; j++) {
			if (j == (int)(unitlength*(log10(points[i]) - ymin))) 
				fprintf(datafile, "*");
			else 
				fprintf(datafile, " ");
		}
		fprintf(datafile, "+   %3d\n", i);
	}

	fclose(datafile);
}

int check_graph(char *buf)
{
	// return true if there "#graph" strings 
	FILE *fp;
	char buffer[BUFSIZE], runname[BUFSIZE], outputfile[BUFSIZE];

	sscanf(buf, "%s", outputfile);
	fp = fopen(outputfile,"r");
	if (fp == NULL) {
		return 1;
	}
	while (1) {
		if (fgets(buffer, sizeof(buffer), fp) == NULL)
			break;

		sscanf(buffer, "%s", runname);
		if (strcmp(runname, "#graph") == 0) {
			fclose(fp);
			return 1;
		}
	}
	fclose(fp);
	return 0;
}

void draw_scale(FILE *datafile, int ymin, int unitlength, int digit)
{
	int j, count;

	fprintf(datafile, "#THETA(CM)");
	for (j = count = 0; j <= unitlength * digit; j++) {
		if (j % unitlength == 0) {
			fprintf(datafile, "%-2d", ymin + count);
			j++;
			count++;
		} else {
			fprintf(datafile, " ");
		}
	}
	fprintf(datafile, "THETA(CM)\n");
}

void draw_axis(FILE *datafile, int deg, int unitlength, int digit)
{
	int j;
	fprintf(datafile, "#      %3d", deg);
	for (j = 0; j <= unitlength * digit; j++) {
		if (j % unitlength == 0)
			fprintf(datafile, "|");
		else
			fprintf(datafile, "+");
	}
	fprintf(datafile, "   %3d\n", deg);
}

// get target number
double get_target_number(int target,int nuclei, int position)
{
	double targetNumber, surfacedensity, massnumber, correction;

	// get surface density and mass number
	if (target == CH2TRGT) {
		surfacedensity = CH2SRFCDNS;
		if (nuclei == CNCL) {
			massnumber = (A_C12+2.0e0*A_H);
		} else if (nuclei == HNCL) {
			massnumber = (A_C12+2.0e0*A_H)/2.0e0;
		} else {
			return 0;
		}
	} else if (target == ALMTRGT) {
		surfacedensity = ALMSRFCDNS;
		if (nuclei == SAME) {
			massnumber = A_Al;
		}
	} else if (target == MLMNTRGT) {
		surfacedensity = MLMSRFCDNS;
		if (nuclei == CNCL) {
			massnumber = (3.0e0*A_C12+6.0e0*A_H+6.0e0*A_N)/3.0e0;
		} else if (nuclei == HNCL) {
			massnumber = (3.0e0*A_C12+6.0e0*A_H+6.0e0*A_N)/6.0e0;
		} else if (nuclei == NNCL) {
			massnumber = (3.0e0*A_C12+6.0e0*A_H+6.0e0*A_N)/6.0e0;
		} else {
			return 0;
		}
	} else if (target == C13TRGT) {
		surfacedensity = C13SRFCDNS;
		if (nuclei == SAME) {
			massnumber = A_C13;
		} else {
			return 0;
		}
	} else {
		return 0;
	}

	if (position == NRMPOSI) {
		correction = 1.0e0;
	} else if (position == ANMPOSI) {
		correction = sqrt(2.0e0);
	} else {
		return 0;
	}

	targetNumber = correction*surfacedensity*NA/massnumber;
	return targetNumber;
}

int get_nuclei_number(int A, int target)
{
	int nuclei_num;
	if (target == C13TRGT) { nuclei_num = SAME; }
	else if (target == ALMTRGT) { nuclei_num = SAME; }
	else if (target == MLMNTRGT) {
		if (A == C12NUM) { nuclei_num = CNCL; }
		else if (A == H1NUM) { nuclei_num = HNCL; }
		else if (A == N14NUM) { nuclei_num = NNCL; }
		else { nuclei_num = 0; }
	} else if (target == CH2TRGT) {
		if (A == C12NUM) { nuclei_num = CNCL; }
		else if (A == H1NUM) { nuclei_num = HNCL; }
		else { nuclei_num = 0; }
	} else { nuclei_num = 0; }
	return nuclei_num;
}

void make_tex_file(char *title)
{
	char textitle[BUFSIZE], target[BUFSIZE];

	// .dat -> .tex
	strncpy(textitle,title, strlen(title)-4);
	textitle[strlen(title)-4] = '\0';
	strcat(textitle,"plus.tex");
	texout = fopen(textitle,"w");
	if (texout == NULL) {
		printf("cannot make texout file\n");
		return;
	}
	strncpy(target, title, strlen(title)-4);
	textitle[strlen(title)-4] = '\0';
	// tex template
	fprintf(texout, "\\documentclass{jsarticle}\n");
	fprintf(texout, "\\usepackage{amsmath}\n");
	fprintf(texout, "\\usepackage[psamsfonts]{amssymb}\n");
	fprintf(texout, "\\usepackage[dvipdfm]{graphicx}\n");
	fprintf(texout, "\\usepackage{okumacro}\n");
	fprintf(texout, "\\usepackage{booktabs}\n");
	fprintf(texout, "\\usepackage{float}\n");
	fprintf(texout, "\\numberwithin{figure}{section}\n");
	fprintf(texout, "\\numberwithin{table}{section}\n");
	fprintf(texout, "\\numberwithin{equation}{section}\n");
	fprintf(texout, "\\renewcommand{\\figurename}{Fig.}\n");
	fprintf(texout, "\\renewcommand{\\tablename}{Table.}\n");
	fprintf(texout, "\\begin{document}\n");
	fprintf(texout, "\\title{Cross Section %s}\n", textitle);
	fprintf(texout, "\\date{}\n\\maketitle\n");
	fprintf(texout, "\\newpage\n");
}
